#!/usr/bin/perl -w
#
# TWiki add-on development support package
#
# Copyright (C) 2002 Esteban Manchado Velzquez, zoso@foton.es
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details, published at 
# http://www.gnu.org/copyleft/gpl.html
#

package TWiki::Node;

use strict;

use lib ( '.' );
use lib ( '../lib' );

# ===========================
# TWiki config variables:
use vars qw(
        $webName $topicName $includingWebName $includingTopicName                       $defaultUserName $userName $wikiName $wikiUserName
        $wikiHomeUrl $defaultUrlHost $urlHost
        $scriptUrlPath $pubUrlPath $viewScript                                          $pubDir $templateDir $dataDir $twikiLibDir
        $siteWebTopicName $wikiToolName $securityFilter $uploadFilter
        $debugFilename $warningFilename $htpasswdFilename                               $logFilename $remoteUserFilename $wikiUsersTopicname
        $userListFilename %userToWikiList %wikiToUserList                               $twikiWebname $mainWebname $mainTopicname $notifyTopicname
        $wikiPrefsTopicname $webPrefsTopicname
        $statisticsTopicname $statsTopViews $statsTopContrib
        $numberOfRevisions $editLockTime
        $attachAsciiPath $scriptSuffix $wikiversion
        $safeEnvPath $mailProgram $noSpamPadding $mimeTypesFilename
        $doKeepRevIfEditLock $doGetScriptUrlFromCgi $doRemovePortNumber                 $doRememberRemoteUser $doPluralToSingular
        $doHidePasswdInRegistration $doSecureInclude
        $doLogTopicView $doLogTopicEdit $doLogTopicSave $doLogRename
        $doLogTopicAttach $doLogTopicUpload $doLogTopicRdiff
        $doLogTopicChanges $doLogTopicSearch $doLogRegistration
        $disableAllPlugins);


# TWiki::Store config:
use vars qw(
        $useRcsDir
        $revInitBinaryCmd $revCoCmd $revCiCmd $revCiDateCmd $revHistCmd
        $revInfoCmd $revDiffCmd $revDelRevCmd $revUnlockCmd $revLockCmd $nullDev
        $rcsArg
    );

# TWiki::Search config:
use vars qw(
        $cmdQuote $lsCmd $egrepCmd $fgrepCmd
    );

# read the configuration part
do "TWiki.cfg";

sub new
{
    my $proto = shift ;
    my $class = ref( $proto ) || $proto;
    my $wiki = shift ;
    my $name = shift ;

    my $self = {};
    $self->{TWIKIROOT} = @_ ? shift : $dataDir;       # Optional argument
    $self->{WIKI} = $wiki;
    $self->{NAME} = $name;
    bless( $self, $class );
    return $self;
}

# ===========================
# Returns the TWiki root directory
sub twikiroot
{
    my $self = shift ;

    return $self->{TWIKIROOT};
}

# ===========================
# Returns the absolute path of the file containing the node (if it exists)
sub filename
{
    my $self = shift ;
    return $self->{TWIKIROOT} . '/' . $self->{WIKI} .
                                '/' . $self->{NAME} . '.txt';
}

# ===========================
# Returns the node text
sub text
{
    my $self = shift ;

    if( exists $self->{TEXT} ) {
        return $self->{TEXT};
    } else {
        my $oldslash = $/;
        undef $/; # set to read to EOF
        my $filename = $self->filename;
        open( IN_FILE, "<$filename" ) || return "";
        $self->{TEXT} = <IN_FILE>;
        $/ = $oldslash;
        close IN_FILE;
    }
    return $self->{TEXT};
}

# ===========================
# Sets the node text
sub setText
{
    my $self = shift ;
    my $theText = shift ;

    $self->{TEXT} = $theText;
    return $theText;
}

# ===========================
# Saves the current text
sub save
{
    my $self = shift ;

    return undef unless exists $self->{TEXT};

    # Save the file
    my $filename = $self->filename;
    open( FILE, ">$filename" ) or warn "Can't create file $filename";
    print FILE $self->{TEXT};
    close FILE;
    # Commit to RCS
    my @time = gmtime time;
    # format to RCS date "2000/12/31/23:59:59"
    my $date = sprintf( "%.4u/%.2u/%.2u/%.2u:%.2u:%.2u", $time[5] + 1900,
                        $time[4] + 1, $time[3], $time[2], $time[1], $time[0] );
    my $cmd = $revCiDateCmd;
    $cmd =~ s/%USERNAME%/$defaultUserName/;
    $cmd =~ s/%FILENAME%/$filename/;
    $cmd =~ s/%DATE%/$date/;
    my $rcserror = `$cmd 2>&1 1>/dev/null`;
    print " $rcserror" if( $rcserror );

    return $self->{TEXT};
}

1;
